﻿
using System;
using System.Collections;
using System.Collections.Generic;
using NHibernate;
using NHibernate.Cfg;


namespace PamNHibernateDemos
{
  public class Program
  {

    private static ISessionFactory sessionFactory = null;

    // programme principal
    static void Main(string[] args)
    {
      // initialisation factory NHibernate
      sessionFactory = new Configuration().Configure().BuildSessionFactory();
      try
      {
        // suppression du contenu de la base
        Console.WriteLine("Effacement base -------------------------------------");
        ClearDataBase1();
        Console.WriteLine("Affichage base -------------------------------------");
        ShowDataBase();
        Console.WriteLine("Remplissage base -------------------------------------");
        FillDataBase();
        Console.WriteLine("Affichage base -------------------------------------");
        ShowDataBase();
        Console.WriteLine("Effacement base -------------------------------------");
        ClearDataBase1();
        Console.WriteLine("Création de deux indemnités de même indice dans une transaction --------------");
        CreateIndemnites1();
        Console.WriteLine("Affichage base -------------------------------------");
        ShowDataBase();
        Console.WriteLine("Création de deux indemnités de même indice sans transaction --------------");
        CreateIndemnites2();
        Console.WriteLine("Affichage base -------------------------------------");
        ShowDataBase();
        Console.WriteLine("Effacement base -------------------------------------");
        ClearDataBase1();
        Console.WriteLine("Remplissage base -------------------------------------");
        FillDataBase();
        Console.WriteLine("Recherche d'un employé -------------------------------------");
        FindEmployee();
				Console.WriteLine("sauvegarde d'un employé vide -------------------------------------");
				SaveEmptyEmployee();
				Console.WriteLine("Affichage base -------------------------------------");
				ShowDataBase();
				Console.WriteLine("sauvegarde d'un employé avec un n° SS trop long -------------------------------------");
				SaveEmployeeWithSS();
				Console.WriteLine("Affichage base -------------------------------------");
				ShowDataBase();
			}
      catch (Exception ex)
      {
        // on affiche l'exception 
        Console.WriteLine(string.Format("L'erreur suivante s'est produite : [{0}]", ex.ToString()));
      }
      finally
      {
        if (sessionFactory != null)
        {
          sessionFactory.Close();
        }
      }
      // attente clavier
      Console.ReadLine();
    }

    // test1
    static void ShowDataBase()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        // début transaction
        using (ITransaction transaction = session.BeginTransaction())
        {
          // on récupère la liste des employés
          IList<Employe> employes = session.CreateQuery(@"select e from Employe e order by e.Nom asc").List<Employe>();
          // on l'affiche
          Console.WriteLine("--------------- liste des employés");
          foreach (Employe e in employes)
          {
            Console.WriteLine(e);
          }
          // on récupère la liste des indemnites
          IList<Indemnites> indemnites = session.CreateQuery(@"select i from Indemnites i order by i.Indice asc").List<Indemnites>();
          // on l'affiche
          Console.WriteLine("--------------- liste des indemnités");
          foreach (Indemnites i in indemnites)
          {
            Console.WriteLine(i);
          }
          // on récupère la liste des cotisations
          Cotisations cotisations = session.CreateQuery(@"select c from Cotisations c").UniqueResult<Cotisations>();
          Console.WriteLine("--------------- tableau des taux de cotisations");
          Console.WriteLine(cotisations);
          // commit transaction
          transaction.Commit();
        }
      }
    }

    // ClearDataBase1
    static void ClearDataBase1()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        // début transaction
        using (ITransaction transaction = session.BeginTransaction())
        {
          // on récupère la liste des employés
          IList<Employe> employes = session.CreateQuery(@"select e from Employe e").List<Employe>();
          // on supprime tous les employés
          Console.WriteLine("--------------- suppression des employés");
          foreach (Employe e in employes)
          {
            session.Delete(e);
          }
          // on récupère la liste des indemnités restantes
          IList<Indemnites> indemnites = session.CreateQuery(@"select i from Indemnites i").List<Indemnites>();
          // on supprime les indemnités
          Console.WriteLine("--------------- suppression des indemnités");
          foreach (Indemnites i in indemnites)
          {
            session.Delete(i);
          }
          // on récupère la liste des cotisations
          Cotisations cotisations = session.CreateQuery(@"select c from Cotisations c").UniqueResult<Cotisations>();
          Console.WriteLine("--------------- suppression des taux de cotisations");
          if (cotisations != null)
          {
            session.Delete(cotisations);
          }
          // commit transaction
          transaction.Commit();
        }
      }
    }

    // FillDataBase
    static void FillDataBase()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        // début transaction
        using (ITransaction transaction = session.BeginTransaction())
        {
          // on crée deux indemnités
          Indemnites i1 = new Indemnites() { Id = 0, Indice = 1, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
          Indemnites i2 = new Indemnites() { Id = 0, Indice = 2, BaseHeure = 2.1, EntretienJour = 2.1, RepasJour = 3.1, IndemnitesCp = 15 };
          // on crée deux employés
          Employe e1 = new Employe() { Id = 0, SS = "254104940426058", Nom = "Jouveinal", Prenom = "Marie", Adresse = "5 rue des oiseaux", Ville = "St Corentin", CodePostal = "49203", Indemnites = i2 };
          Employe e2 = new Employe() { Id = 0, SS = "260124402111742", Nom = "Laverti", Prenom = "Justine", Adresse = "La Brûlerie", Ville = "St Marcel", CodePostal = "49014", Indemnites = i1 };
          // on crée les taux de cotisations
          Cotisations cotisations = new Cotisations() { Id = 0, CsgRds = 3.49, Csgd = 6.15, Secu = 9.39, Retraite = 7.88 };
          // on sauvegarde le tout
          session.SaveOrUpdate(e1);
					session.SaveOrUpdate(e2);
					session.SaveOrUpdate(cotisations);
          // commit transaction
          transaction.Commit();
        }
      }
    }

    // CreateIndemnites1
    static void CreateIndemnites1()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        // début transaction
        try
        {
          using (ITransaction transaction = session.BeginTransaction())
          {
            // on crée deux indemnités de même indice
            Indemnites i1 = new Indemnites() { Id = 0, Indice = 1, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
            Indemnites i2 = new Indemnites() { Id = 0, Indice = 1, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
            // on les sauvegarde
            session.Save(i1);
            session.Save(i2);
            // commit transaction
            transaction.Commit();
          }
        }
        catch (Exception e)
        {
          Console.WriteLine("L'exception suivante s'est produite : " + e.Message);
        }
      }
    }

    // CreateIndemnites2
    static void CreateIndemnites2()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        try
        {
          // on crée deux indemnités de même indice
          Indemnites i1 = new Indemnites() { Id = 0, Indice = 1, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
          Indemnites i2 = new Indemnites() { Id = 0, Indice = 1, BaseHeure = 1.94, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
          // on les sauvegarde
          session.Save(i1);
          session.Save(i2);
        }
        catch (Exception e)
        {
          Console.WriteLine("L'exception suivante s'est produite : " + e.Message);
        }
      }
    }

    // FindEmployee
    static void FindEmployee()
    {
      // ouverture session 
      using (ISession session = sessionFactory.OpenSession())
      {
        // début transaction
        try
        {
          using (ITransaction transaction = session.BeginTransaction())
          {
            // on recherche un employé à partir de son n° SS
            String numSecu = "254104940426058";
            IQuery query=session.CreateQuery(@"select e from Employe e where e.SS=:numSecu");
            Employe employe = query.SetString("numSecu", numSecu).UniqueResult<Employe>();
            if (employe != null)
            {
              Console.WriteLine("Employe[" + numSecu + "]=" + employe);
            }
            else
            {
              Console.WriteLine("Employe[" + numSecu + "] non trouvé...");
            }

            numSecu = "xx";
            employe = query.SetString("numSecu", numSecu).UniqueResult<Employe>();
            if (employe != null)
            {
              Console.WriteLine("Employe[" + numSecu + "]=" + employe);
            }
            else
            {
              Console.WriteLine("Employe[" + numSecu + "] non trouvé...");
            }

            // commit transaction
            transaction.Commit();
          }
        }
        catch (Exception e)
        {
          Console.WriteLine("L'exception suivante s'est produite : " + e.Message);
        }
      }
    }

		// SaveEmptyEmployee
		static void SaveEmptyEmployee() {
			// ouverture session 
			using (ISession session = sessionFactory.OpenSession()) {
				// début transaction
				try {
					using (ITransaction transaction = session.BeginTransaction()) {
						// on crée un employe vide
						Employe e = new Employe();
						// on crée une indemnité non existante
						Indemnites i = new Indemnites() { Id = 0, Indice = 3, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
						// qu'on associe à l'employé
						e.Indemnites = i;
						// on sauvegarde l'employé en laissant vides les autres champs
						session.Save(e);
						// commit transaction
						transaction.Commit();
					}
				} catch (Exception e) {
					Console.WriteLine("L'exception suivante s'est produite : " + e.Message);
				}
			}
		}

		// SaveEmployeeWithSS
		static void SaveEmployeeWithSS() {
			// ouverture session 
			using (ISession session = sessionFactory.OpenSession()) {
				// début transaction
				try {
					using (ITransaction transaction = session.BeginTransaction()) {
						// on crée une indemnité non existante
						Indemnites i = new Indemnites() { Id = 0, Indice = 3, BaseHeure = 1.93, EntretienJour = 2, RepasJour = 3, IndemnitesCp = 12 };
						// on crée un employe vide avec un n° SS trop grand
						Employe e = new Employe() { Id = 0, SS = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx", Nom = "Jouveinal", Prenom = "Marie", Adresse = "5 rue des oiseaux", Ville = "St Corentin", CodePostal = "49203", Indemnites = i };
						// on sauvegarde l'employé
						session.Save(e);
						// commit transaction
						transaction.Commit();
					}
				} catch (Exception e) {
					Console.WriteLine("L'exception suivante s'est produite : " + e);
				}
			}
		}

  }
}
