﻿using System;
using System.Collections.Generic;
using Pam.Dao.Entites;
using Spring.Transaction.Interceptor;
using Spring.Data.NHibernate.Generic.Support;
using System.Collections;

namespace Pam.Dao.Service {
	public class PamDaoSpringNHibernate : HibernateDaoSupport, IPamDao {
		// champs privés 
		private Cotisations cotisations;
		private Employe[] employes;

		// init 
		[Transaction(ReadOnly = true)]
		public void init() {
			try {
				// on récupère la liste simplifiée des employés
				IList<object[]> lignes = HibernateTemplate.Find<object[]>("select e.SS,e.Nom,e.Prenom from Employe e");
				// on la met dans un tableau
				employes = new Employe[lignes.Count];
				int i = 0;
				foreach (object[] ligne in lignes) {
					employes[i] = new Employe() { SS = ligne[0].ToString(), Nom = ligne[1].ToString(), Prenom = ligne[2].ToString() };
					i++;
				}
				// on met les taux de cotisations dans un objet 
				cotisations = (HibernateTemplate.LoadAll<Cotisations>())[0] as Cotisations;
			} catch (Exception ex) {
				// on transforme l'exception 
				throw new PamException(string.Format("Erreur d'accès à la BD : [{0}]", ex.ToString()), 43);
			}
		}

		// suppression objet
		public void destroy() {
			if (HibernateTemplate.SessionFactory != null) {
				HibernateTemplate.SessionFactory.Close();
			}
		}

		// liste de toutes les identités des employés
		[Transaction(ReadOnly = true)]
		public Employe[] GetAllIdentitesEmployes() {
			return employes;
		}


		// un employé particulier avec ses indemnités 
		[Transaction(ReadOnly = true)]
		public Employe GetEmploye(string ss) {
			IList<Employe> employés = null;
			try {
				// requête
				employés = HibernateTemplate.Find<Employe>("select e from Employe e where e.SS=?", ss) as IList<Employe>;
			} catch (Exception ex) {
				// on transforme l'exception 
				throw new PamException(string.Format("Erreur d'accès à la BD lors de la demande de l'employé de n° ss [{0}] : [{1}]", ss, ex.ToString()), 41);
			}
			// a-t-on récupéré un employé ? 
			if (employés.Count == 0) {
				// on signale le fait 
				throw new PamException(string.Format("L'employé de n° ss [{0}] n'existe pas", ss), 42);
			} else {
				return employés[0];
			}
		}

		// liste des cotisations 
		[Transaction(ReadOnly = true)]
		public Cotisations GetCotisations() {
			return cotisations;
		}
	}
}
