﻿using System.Collections;
using NUnit.Framework;
using Pam.Dao.Service;
using Pam.Dao.Entites;
using Spring.Objects.Factory.Xml;
using Spring.Core.IO;
using Spring.Context.Support;

namespace Pam.Dao.Tests {

	[TestFixture]
	public class NunitPamDao : AssertionHelper {
		// la couche [dao] à tester 
		private IPamDao pamDao = null;

		// constructeur 
		public NunitPamDao() {
			// instanciation couche [dao]
			pamDao = (IPamDao)ContextRegistry.GetContext().GetObject("pamdao");
		}

		// init 
		[SetUp]
		public void Init() {

		}

		[Test]
		public void GetAllIdentitesEmployes() {
			// vérification nbre d'employes 
			Expect(2, EqualTo(pamDao.GetAllIdentitesEmployes().Length));
		}

		[Test]
		public void GetCotisations() {
			// vérification taux de cotisations 
			Cotisations cotisations = pamDao.GetCotisations();
			Expect(3.49, EqualTo(cotisations.CsgRds).Within(1E-06));
			Expect(6.15, EqualTo(cotisations.Csgd).Within(1E-06));
			Expect(9.39, EqualTo(cotisations.Secu).Within(1E-06));
			Expect(7.88, EqualTo(cotisations.Retraite).Within(1E-06));
		}

		[Test]
		public void GetEmployeIdemnites() {
			// vérification individus 
			Employe employe1 = pamDao.GetEmploye("254104940426058");
			Employe employe2 = pamDao.GetEmploye("260124402111742");
			Expect("Jouveinal", EqualTo(employe1.Nom));
			Expect(2.1, EqualTo(employe1.Indemnites.BaseHeure).Within(1E-06));
			Expect("Laverti", EqualTo(employe2.Nom));
			Expect(1.93, EqualTo(employe2.Indemnites.BaseHeure).Within(1E-06));
		}

		[Test]
		public void GetEmployeIdemnites2() {
			// vérification individu inexistant 
			bool erreur = false;
			try {
				Employe employe1 = pamDao.GetEmploye("xx");
			} catch {
				erreur = true;
			}
			Expect(erreur, True);
		}
	}
}